@@ -4,6 +4,7 @@ import logging
 import os
 import re
 
+import itsdangerous
 import royalnet.campaigns
 import sqlalchemy.orm
 import telethon
@@ -24,21 +25,50 @@ email_regex = re.compile(r"^([0-9]+)(?:@studenti\.unimore\.it)?$")
 
 class Dialog:
     def __init__(self, bot: telethon.TelegramClient, entity: Entity, session: sqlalchemy.orm.Session):
+        """
+        Initialize an Dialog object.
+
+        .. warning:: Do not use this, use the Dialog.create() method instead!
+        """
         self.bot: telethon.TelegramClient = bot
+        "The bot at one side of the Dialog."
+
         self.entity: Entity = entity
+        "The entity (user, group) at the other side of the Dialog."
+
         self.session: sqlalchemy.orm.Session = session
+        "The SQLAlchemy Session to be used exclusively for this dialog."
+
         self.campaign: royalnet.campaigns.AsyncCampaign = ...
+        """
+        The AsyncCampaign this dialog will use to track the back and forth (taking form of multiple AsyncAdventures).
+        
+        May be an instance of Ellipsis if the Dialog is not initialized.
+        """
 
     @classmethod
     async def create(cls,
                      bot: telethon.TelegramClient,
                      entity: Entity,
                      session: sqlalchemy.orm.Session) -> Dialog:
+        """
+        Create a new Dialog object.
+
+        :param bot: The bot at one side of the Dialog.
+        :param entity: The entity (user, group) at the other side of the Dialog.
+        :param session: The SQLAlchemy Session to be used exclusively for this dialog.
+        :return: The created dialog.
+        """
         menu = cls(bot=bot, entity=entity, session=session)
         menu.campaign = await royalnet.campaigns.AsyncCampaign.create(start=menu.__first())
         return menu
 
     async def next(self, msg: telethon.tl.custom.Message) -> None:
+        """
+        Advance to the next message of the Dialog.
+
+        :param msg: The message to pass to the AsyncAdventure.
+        """
         try:
             log.debug(f"Advancing: {self}")
             await self.campaign.next(msg)
@@ -50,8 +80,16 @@ class Dialog:
             log.debug(f"Sending: {self.campaign.challenge}")
             await self.campaign.challenge.send(bot=self.bot, entity=self.entity)
 
-    async def __message(self, msg, **kwargs):
-        """Send a message to the specified entity."""
+    async def __message(self, msg, **kwargs) -> telethon.types.Message:
+        """
+        Send a Telegram message to the user at the other side of the dialog.
+
+        It's a shortcut to self.bot.send_message.
+
+        :param msg: The contents of the message.
+        :param kwargs: Keyword arguments to pass to send_message.
+        :return: The Message returned by send_message.
+        """
         return await self.bot.send_message(
             entity=self.entity,
             parse_mode="HTML",
@@ -61,13 +99,13 @@ class Dialog:
         )
 
     async def __first(self) -> AsyncAdventure:
-        """The generator which chooses the first dialog."""
+        """What to do when the dialog is first opened."""
         msg: telethon.tl.custom.Message = yield
 
         # If this is a text message
         if text := msg.message:
             if text.startswith("/whois"):
-                yield self.__whois(text=text)
+                yield self.__whois()
 
             elif text.startswith("/start"):
                 if msg.is_private:
@@ -85,11 +123,23 @@ class Dialog:
                         f"⚠️ Questo comando funziona solo in chat privata (@{os.environ['TELEGRAM_BOT_USERNAME']})."
                     )
 
-    async def __start(self):
+    async def __start(self) -> AsyncAdventure:
+        """Disambiguation for the /start command."""
         msg: telethon.tl.custom.Message = yield
 
         text: str = msg.message
 
+        # Check if the user is already registered
+        from_user = await msg.get_sender()
+        tg: Telegram = self.session.query(Telegram).filter_by(id=from_user.id).one_or_none()
+        if tg is not None:
+            await self.__message(
+                f'⭐️ Hai già effettuato la verifica dell\'identità.\n\n'
+                f'<a href="{os.environ["GROUP_URL"]}">Entra nel gruppo cliccando '
+                f'qui!</a>'
+            )
+            return
+
         # Check whether this is a normal start or a deep-linked one
         split = text.split(" ", 1)
         if len(split) == 1:
@@ -97,26 +147,43 @@ class Dialog:
         else:
             yield self.__deeplink_start(payload=split[1])
 
-    async def __normal_start(self):
+    async def __normal_start(self) -> AsyncAdventure:
+        """The /start command, called without arguments."""
         msg: telethon.tl.custom.Message = yield
 
         await self.__message(
-            f'👋 Ciao! Sono Thor, il bot-moderatore di Unimore Informatica.\n\n'
-            f'Per entrare nel gruppo devi <a href="{os.environ["BASE_URL"]}/">effettuare la verifica '
-            f'dell\'identità facendo il login qui con il tuo account Unimore</a>.\n\n'
-            f'Se hai bisogno di aiuto, manda un messaggio a @Steffo.'
+            f'👋 Ciao! Sono Thor, il bot-moderatore di Unimore Informatica.\n'
+            f'\n'
+            f'Se vuoi entrare nel gruppo, devi <b>dimostrare di essere uno studente dell\'Unimore</b>.\n'
+            f'\n'
+            f'<a href="{os.environ["BASE_URL"]}">Fai il login con il tuo account universitario qui</a>, poi una volta '
+            f'tornato su Telegram premi il tasto <i>AVVIA</i> in basso per ricevere il link! 😊'
         )
 
-    async def __deeplink_start(self, payload: str):
+    async def __deeplink_start(self, payload: str) -> AsyncAdventure:
+        """The /start command, called with deep-linked arguments."""
         msg: telethon.tl.custom.Message = yield
 
-        opcode, data = dl.decode(payload)
+        try:
+            opcode, data = dl.decode(payload)
+        except itsdangerous.exc.BadData:
+            await self.__message("⚠️ I dati ricevuti non sono validi.")
+            return
 
         # R: Register new account
         if opcode == "R":
             yield self.__register(email_prefix=data)
+        else:
+            await self.__message("⚠️ Ricevuto un opcode sconosciuto.")
 
     async def __register(self, email_prefix: str) -> AsyncAdventure:
+        """
+        The /start command, called with a payload starting with R.
+
+        Links a Telegram account to a real student.
+
+        :param email_prefix: The prefix before @studenti.unimore.it
+        """
         msg: telethon.tl.custom.Message = yield
 
         from_user = await msg.get_sender()
@@ -124,27 +191,10 @@ class Dialog:
         tg: Telegram = self.session.query(Telegram).filter_by(id=from_user.id).one_or_none()
         st: Student = self.session.query(Student).filter_by(email_prefix=email_prefix).one()
 
-        if tg is not None:
-            # The user is already registered
-            if tg.st == st:
-                await self.__message(
-                    f'⭐️ Hai già effettuato la verifica dell\'identità.\n\n'
-                    f'<a href="{os.environ["GROUP_URL"]}">Entra nel gruppo cliccando '
-                    f'qui!</a>'
-                )
-                return
-            # The account is connected to someone else
-            else:
-                await self.__message(
-                    f"⚠️ Questo account Telegram è già connesso a <b>{tg.st.first_name} {tg.st.last_name}"
-                    f"</b>.\n\n"
-                )
-                return
-
         # Ask for confirmation
         choice = yield Keyboard(
             message=f'❔ Tu sei {st.first_name} {st.last_name} <{st.email()}>, giusto?',
-            choices=[["✅ Sì!", "❌ No."]]
+            choices=[["❌ No.", "✅ Sì!"]]
         )
         if choice.message == "❌ No.":
             await self.__message(
@@ -154,15 +204,13 @@ class Dialog:
 
         # Ask for privacy mode
         choice = yield Keyboard(
-            message="📝 Vuoi aggiungere il tuo nome e la tua email alla rubrica del gruppo?\n"
-                    "\n"
-                    "Questo li renderà visibili a tutti gli altri membri verificati.\n"
-                    "\n"
+            message="📝 Vuoi permettere agli altri studenti verificati di visualizzare il tuo <b>vero nome</b> e la tua "
+                    "<b>email istituzionale</b> attraverso il comando /whois?\n\n"
                     "(Gli amministratori del gruppo vi avranno comunque accesso, e potrai cambiare idea in qualsiasi "
                     "momento con il comando /privacy.)",
-            choices=[["✅ Sì!", "❌ No."]]
+            choices=[["👤 Nascondi.", "📱 Mostra!"]]
         )
-        privacy = choice.message == "❌ No."
+        st.privacy = choice.message == "👤 Nascondi."
 
         # Create the SQL record
         tg = Telegram(
@@ -170,7 +218,6 @@ class Dialog:
             first_name=from_user.first_name,
             last_name=from_user.last_name,
             username=from_user.username,
-            privacy=privacy,
             st=st,
         )
 
@@ -185,7 +232,8 @@ class Dialog:
         )
         return
 
-    async def __privacy(self):
+    async def __privacy(self) -> AsyncAdventure:
+        """The /privacy command, used to toggle between privacy states."""
         msg: telethon.tl.custom.Message = yield
 
         from_user = await msg.get_sender()
@@ -201,24 +249,25 @@ class Dialog:
 
         # Ask for privacy mode
         choice = yield Keyboard(
-            message="📝 Vuoi aggiungere il tuo nome e la tua email alla rubrica del gruppo?\n\n"
-                    "Questo li renderà visibili a tutti gli altri membri verificati.\n\n"
+            message="📝 Vuoi permettere agli altri studenti verificati di visualizzare il tuo <b>vero nome</b> e la tua "
+                    "<b>email istituzionale</b> attraverso il comando /whois?\n\n"
                     "(Gli amministratori del gruppo vi avranno comunque accesso, e potrai cambiare idea in qualsiasi "
                     "momento con il comando /privacy.)",
-            choices=[["✅ Sì!", "❌ No."]]
+            choices=[["👤 Nascondi.", "📱 Mostra!"]],
         )
-        tg.privacy = choice.message == "❌ No."
+        tg.st.privacy = choice.message == "👤 Nascondi."
         self.session.commit()
 
-        if tg.privacy:
-            await self.__message("❌ I tuoi dati ora sono nascosti dalla rubrica del gruppo.")
+        if tg.st.privacy:
+            await self.__message("👤 I tuoi dati ora sono nascosti.")
         else:
-            await self.__message("✅ I tuoi dati ora sono visibili nella rubrica del gruppo!")
+            await self.__message("📱 I tuoi dati ora sono visibili attraverso il comando /whois!")
 
-    async def __whois(self, text: str):
+    async def __whois(self) -> AsyncAdventure:
+        """The /whois command, used to fetch information about a certain student or Telegram account."""
         msg: telethon.tl.custom.Message = yield
 
-        cmd, *args = text.split(" ", 1)
+        cmd, *args = msg.message.split(" ", 1)
         args = " ".join(args)
 
         # Email
@@ -234,72 +283,65 @@ class Dialog:
             username = args.lstrip("@")
             yield self.__whois_username(username=username)
 
+        # TODO: Match telegram mentions
+        # TODO: Match telegram name
+
         await self.__message(
             "⚠️ Non hai specificato correttamente cosa cercare.\n"
             "\n"
-            "Puoi specificare un'username Telegram, un nome e cognome o un'email."
+            "Puoi specificare un'username Telegram, un nome e cognome o un'email.\n"
+            "<code>/whois Stefano Pigozzi</code>\n"
+            "<code>/whois @Steffo</code>\n"
+            "<code>/whois 256895@studenti.unimore.it</code>\n"
+            "\n"
+            "🚧 La funzionalità di ricerca tramite name-mention di Telegram non è ancora stata implementata."
         )
 
     async def __whois_email(self, email_prefix: str):
+        """The /whois command, called with an email."""
         msg: telethon.tl.custom.Message = yield
 
-        result = self.session.query(Student).filter_by(email_prefix=email_prefix).one_or_none()
-        if result is None:
+        st: Optional[Student] = self.session.query(Student).filter_by(email_prefix=email_prefix).one_or_none()
+        if st is None:
             await self.__message("⚠️ Nessuno studente trovato.")
-        elif result.tg.privacy:
-            await self.__message(
-                "👤 Lo studente è registrato, ma ha deciso di manterere privati i dettagli del suo account."
-            )
-        else:
-            await self.__message(result.message())
+            return
 
-    async def __whois_real_name(self, name: str):
+        await self.__message(st.whois())
+
+    async def __whois_real_name(self, name: str) -> AsyncAdventure:
+        """The /whois command, called with a first name and a last name."""
         msg: telethon.tl.custom.Message = yield
 
-        sq = (
+        students = (
             self.session
-            .query(
-                Student,
-                sqlalchemy.func.concat(Student.first_name, " ", Student.last_name).label("full_name")
+            .query(Student)
+            .filter(
+                sqlalchemy.or_(
+                    sqlalchemy.func.concat(Student.first_name, " ", Student.last_name) == name.upper(),
+                    sqlalchemy.func.concat(Student.last_name, " ", Student.first_name) == name.upper(),
+                )
             )
-            .subquery()
-        )
-        result = (
-            self.session
-            .query(sq)
-            .filter_by(full_name=name.upper())
             .all()
         )
 
-        if len(result) == 0:
+        if len(students) == 0:
             await self.__message("⚠️ Nessuno studente trovato.")
             return
 
         # There might be more than a student with the same name!
         response: List[str] = []
-        hidden: bool = False
-        for student in result:
-            if student.tg.privacy:
-                hidden = True
-                continue
-            response.append(student.message())
-        if hidden:
-            response.append(
-                "👤 Almeno uno studente ottenuto dalla ricerca è registrato, ma ha deciso di mantenere privati i "
-                "dettagli del suo account."
-            )
+        for student in students:
+            response.append(student.whois())
 
         await self.__message("\n\n".join(response))
 
-    async def __whois_username(self, username: str):
+    async def __whois_username(self, username: str) -> AsyncAdventure:
+        """The /whois command, called with a Telegram username."""
         msg: telethon.tl.custom.Message = yield
 
-        result = self.session.query(Telegram).filter_by(username=username).one_or_none()
-        if result is None:
+        tg: Optional[Telegram] = self.session.query(Telegram).filter_by(username=username).one_or_none()
+        if tg is None:
             await self.__message("⚠️ Nessuno studente trovato.")
-        elif result.privacy:
-            await self.__message(
-                "👤 Lo studente è registrato, ma ha deciso di manterere privati i dettagli del suo account."
-            )
-        else:
-            await self.__message(result.st.message())
+            return
+
+        await self.__message(tg.st.whois())
